#!/bin/sh
# $Id: t-greyfull-0.sh,v 1.5 2010/02/16 17:01:37 ca Exp $
#
# ----------------------------------------
# test "full" greylisting
# uses smtpcnf.sh.
# ----------------------------------------
#

# this is a "slow" test?
if test X"${MTA_NO_SLOW_TEST}" != "X"
then
  echo "SKIPPED: $0: MTA_NO_SLOW_TEST is set"
  exit 0
fi

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../chkmts
fi
. ${SD}/common.sh

SCRIPTS=`cd ${SD} && pwd`
SRCPRG=${STE}/smtpc2

if test X"`uname`" != "XOSF1"
then
while getopts s:V FLAG
do
  case "${FLAG}" in
    s) SKIP="${SKIP} ${OPTARG}";;
    V) VERBOSE=true;;
  esac
done
shift `expr ${OPTIND} - 1`
fi

# cd to right directory (where the programs will be executed).
cd ../qmgr || exit 1

# create empty access map
rm -f ${ACCMAP}
${MM} -t'	' -F ${ACCMAP} < /dev/null

${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar { Log_Level = 14; nameserver = ${NS};
greylisting { grey_wait=10s; grey_expire=20s; white_timeout=15s; }
}
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
Log_Level = 14;
flags = {access,fullgreylisting};
daemon_address = localhost:${SRVPORT};
relay_from = "no.host";
relay_to = "local.host";
wait_for_server=4;
}
EOF

# create aliases
${CAT} > ${ALI} <<EOF
abuse:	local:
postmaster:	local:
user:	local:
ok:	local:
EOF
# create map
rm -f ${ALIMAP}
${MM} < ${ALI}

${CAT} > ${MT} <<EOF
local.host	[127.0.0.1]
y.z		[127.0.0.1]
local.domain	lmtp:
EOF

# ----------------------------------------
# try to send a message: greylisted twice, then accepted
TEST=1
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
rm -f grey_grey_m.db grey_grey_s.db
ERR=false
SENDER=sender${TEST}@y.z
RCPT="-Ruser@local.domain"
if ${SHELL} ${SD}/smtpcnf.sh -a '-d 6' -z -S${SENDER} ${RCPT} -E 0 -X 0 -A 0 -s 2
then
  sleep 1
  if ${SRCPRG} -r ${H}:${SRVPORT} -f"${SENDER}" ${RCPT} > ${CLTL} 2>&1
  then
    # check that no mail has been sent
    if ${GREP} "total=0 " ${CLTL} >/dev/null
    then
      :
    else
      ${VERBOSE} && echo "$0: test ${TEST}: missing total=0 in log"
      ERR=true
    fi
  else
    :
  fi

  sleep 11
  if ${SRCPRG} -r ${H}:${SRVPORT} -f"${SENDER}" ${RCPT} > ${CLTL} 2>&1
  then
    # check that mail has been sent
    if ${GREP} "total=1 " ${CLTL} >/dev/null
    then
      :
    else
      ${VERBOSE} && echo "$0: test ${TEST}: missing total=1 in log"
      ERR=true
    fi
  else
    :
  fi

else
  ${VERBOSE} && echo "$0: test ${TEST}: smtpcnf.sh failed"
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
else
   ${VERBOSE} && echo "PASS: ${TEST}"
fi

${SD}/stopmts.sh
fi


# ----------------------------------------
# try to send a message: greylisted twice, then accepted
TEST=2
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
rm -f grey_grey_m.db grey_grey_s.db
ERR=false
SENDER=sender${TEST}@y.z
RCPT="-Runknown@local.domain -Ruser@local.domain -Rok@local.domain"
if ${SHELL} ${SD}/smtpcnf.sh -z -S${SENDER} ${RCPT} -E 0 -X 0 -A 0 -s 2
then
  sleep 1
  if ${SRCPRG} -r ${H}:${SRVPORT} -f"${SENDER}" ${RCPT} > ${CLTL} 2>&1
  then
    # check that no mail has been sent
    if ${GREP} "total=0 " ${CLTL} >/dev/null
    then
      :
    else
      ${VERBOSE} && echo "$0: test ${TEST}: missing total=0 in log"
      ERR=true
    fi
  else
    :
  fi

  sleep 11
  if ${SRCPRG} -r ${H}:${SRVPORT} -f"${SENDER}" ${RCPT} > ${CLTL} 2>&1
  then
    # check that mail has been sent
    if ${GREP} "total=1 " ${CLTL} >/dev/null
    then
      :
    else
      ${VERBOSE} && echo "$0: test ${TEST}: missing total=1 in log"
      ERR=true
    fi
  else
    :
  fi

else
  ${VERBOSE} && echo "$0: test ${TEST}: smtpcnf.sh failed"
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
else
   ${VERBOSE} && echo "PASS: ${TEST}"
fi

${SD}/stopmts.sh
fi


# ----------------------------------------
# delay_greylisting_error_until_DATA
# try to send a message: greylisted twice, then accepted
TEST=3
if echo "${SKIP} X" | ${GREP} -v " ${TEST} " > /dev/null
then
${CAT} > ${SMXCNF} <<EOF
qmgr { Log_Level = 12; wait_for_server=4; wait_for_client=4; }
smar { Log_Level = 14; nameserver = ${NS};
greylisting { grey_wait=10s; grey_expire=20s; white_timeout=15s; }
}
smtpc { Log_Level = 12; remote_port=${SNKPORT}; wait_for_server=4; }
smtps {
Log_Level = 14;
flags = {access,fullgreylisting,delay_greylisting_error_until_DATA};
daemon_address = localhost:${SRVPORT};
relay_from = "no.host";
relay_to = "local.host";
wait_for_server=4;
}
EOF
rm -f grey_grey_m.db grey_grey_s.db
ERR=false
SENDER=sender${TEST}@y.z
RCPT="-Ruser@local.domain"
if ${SHELL} ${SD}/smtpcnf.sh -z -S${SENDER} ${RCPT} -E 0 -X 0 -A 0 -s 2
then
  sleep 1
  if ${SRCPRG} -r ${H}:${SRVPORT} -f"${SENDER}" ${RCPT} > ${CLTL} 2>&1
  then
    # check that no mail has been sent
    if ${GREP} "total=0 " ${CLTL} >/dev/null
    then
      if ${GREP} "func=ss_data, .* stat=421" ${SSL} >/dev/null
      then
        :
      else
        ${VERBOSE} && echo "$0: test ${TEST}: missing 421 for DATA in log"
        ERR=true
      fi
    else
      ${VERBOSE} && echo "$0: test ${TEST}: missing total=0 in log"
      ERR=true
    fi
  else
    :
  fi

  sleep 11
  if ${SRCPRG} -r ${H}:${SRVPORT} -f"${SENDER}" ${RCPT} > ${CLTL} 2>&1
  then
    # check that mail has been sent
    if ${GREP} "total=1 " ${CLTL} >/dev/null
    then
      :
    else
      ${VERBOSE} && echo "$0: test ${TEST}: missing total=1 in log"
      ERR=true
    fi
  else
    :
  fi

else
  ${VERBOSE} && echo "$0: test ${TEST}: smtpcnf.sh failed"
  ERR=true
fi
if ${ERR}
then
   echo "$0: test ${TEST} failed"
   ERRS=`expr ${ERRS} + 1 `
   test X"${MTA_STOPONERROR}" != X && exit 1
else
   ${VERBOSE} && echo "PASS: ${TEST}"
fi

${SD}/stopmts.sh
fi


# ----------------------------------------
rm -f ${ACCMAP} ${ALIMAP} ${MTMAP}

# ----------------------------------------
# end
if test ${ERRS} -eq 0
then
  exit 0
else
  echo "$0: ${ERRS} error(s)"
  exit 1
fi
