/*
 * Copyright (c) 2010 Sendmail, Inc. and its suppliers.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 *	$Id: dkim.h,v 1.21 2011/01/17 16:06:00 ca Exp $
 */

#ifndef SM_DKIM_H
#define SM_DKIM_H 1

#if MTA_USE_DKIM
#include "sm/generic.h"
#include "sm/sha.h"
#include "sm/io.h"
#include "sm/str.h"
#include "sm/units.h"
#include "sm/queue.h"
#include "sm/time.h"
#include "sm/dkimcnf.h"
#include "sm/map-str.h"

/* check for this? */
#include <openssl/ssl.h>
#include <openssl/err.h>
#include <openssl/bio.h>
#include <openssl/pem.h>

struct dkim_sha1_S
{
	SHA_CTX			sha1_ctx;
	uchar			sha1_out[SHA_DIGEST_LENGTH];
};
typedef struct dkim_sha1_S dkim_sha1_T, *dkim_sha1_P;

struct dkim_sha256_S
{
	SHA256_CTX		sha256_ctx;
	uchar			sha256_out[SHA256_DIGEST_LENGTH];
};
typedef struct dkim_sha256_S dkim_sha256_T, *dkim_sha256_P;

union dkim_sha_U
{
	dkim_sha1_T		dkim_sha1;
	dkim_sha256_T	dkim_sha256;
};
typedef union dkim_sha_U dkim_sha_T, *dkim_sha_P;

TAILQ_HEAD(dkhhd_S, dkh_S);
typedef	struct dkhhd_S	dkhhd_T, *dkhhd_P;

struct dkh_S
{
	uint				 dkh_flags;
	sm_str_P			 dkh_hdr_n;	/* header name */
	sm_str_P			 dkh_hdr_v;	/* header value */
	TAILQ_ENTRY(dkh_S)	 dkh_next;
};
typedef	struct dkh_S	dkh_T, *dkh_P;

/* need to use tailq to allow removal of last element?? */
#define DKH_INIT(dkhhd)	TAILQ_INIT(dkhhd)
#define DKH_FIRST(dkhhd)	\
	(((dkhhd) != NULL) ? TAILQ_FIRST(dkhhd) : NULL)
#define DKH_EMPTY(dkhhd)	\
	((NULL == (dkhhd)) || TAILQ_EMPTY(dkhhd))
#define DKH_APP(dkhhd, dkh)	TAILQ_INSERT_TAIL(dkhhd, dkh, dkh_next)

#define DKH_REMOVE(dkhhd)	TAILQ_REMOVE((dkhhd), DKH_FIRST(dkhhd), dkh_next)
#define DKH_FOREACH_REV(dkh, dkhhd)	TAILQ_FOREACH_REVERSE(dkh, dkhhd, dkhhd_S, dkh_next)

/* DKIM context (handle) */
typedef struct dkim_ctx_S	dkim_ctx_T, *dkim_ctx_P;
typedef struct dkim_msg_S		dkim_msg_T, *dkim_msg_P;

/* context per msg */
struct dkim_msg_S
{
	dkim_ctx_P	 dkm_ctx;
	dkim_s_P	 dkm_s;			/* other signing specifications */
	bool		 dkm_sha256;	/* signing method */

	sm_ret_T	 dkm_stat;

	uint		 dkm_eot_state;
	uint		 dkm_eoh_state;
	int			 dkm_hdr_state;
	uint		 dkm_el_state;
	uint		 dkm_wsp;		/* counter for white space */
	uint		 dkm_elines;	/* number of empty lines encountered */
	sm_file_T	*dkm_hfp;
	sm_file_T	*dkm_bfp;

	bool		 dkm_parsefrom;	/* need to parse From: header? */
	int			 dkm_fhdr_state;	/* From: header state */
	sm_str_P	 dkm_fromhdr;	/* From: header (value) */

	/* should use str ! */
	uchar		 dkm_b64bhash[SHA256_DIGEST_LENGTH * 3];	/* best size? */
	sm_str_P	 dkm_hdr_n;	/* current header name */
	sm_str_P	 dkm_hdr_v;	/* current header value */
	sm_str_P	 dkm_hdrs;	/* list of header names */
	sm_str_P	 dkm_sighdr;	/* DKIM header */
	bool		 dkm_skiphdr;	/* skip current header? */

	uint		 dkm_flags;
	time_T		 dkm_time;
	size_t		 dkm_rsaoutlen;
	uchar		*dkm_rsaout;

	uint32_t	 dkm_hdrtlen;

	dkhhd_T		 dkm_hhd;	/* head of header list */
	dkh_P		 dkm_dkh;
};

#define DKIM_PARSEFROM(dkim_msg)	((dkim_msg)->dkm_parsefrom)

struct dkim_ctx_S
{
	sm_magic_T	 sm_magic;
	dkim_cnf_P	 dkc_cnf;

	dkim_s_P	 dkc_dk_s_dflt;	/* default signing specification */
	sm_map_P	 dkc_dkim_map;	/* map of  signing specifications */
};

extern sm_stream_T SmDKIMSt;

#define dkim_open_write(info, hash, pfp)	\
	sm_io_open(&SmDKIMSt, info, SM_IO_WRONLY, pfp, SM_IO_WHAT_HASH, hash, SM_IO_WHAT_END)

#define dkim_write(fp, buf, buflen, byteswritten)	\
	sm_io_write(fp, buf, buflen, byteswritten)

/* should be a function so it can return an error? */
/* use sizeof() instead of macros! */
#define dkim_gethash(fp, hash, phashlen) do { \
		sm_io_flush(fp);	\
		sm_io_getinfo(fp, SM_IO_WHAT_HASH, hash);	\
		*phashlen = (dkim_cookie->dkf_sha256) ? SHA256_DIGEST_LENGTH : SHA_DIGEST_LENGTH; \
	} while (0)

#define dkim_close(fp, flags) \
	sm_io_close(fp, flags)

sm_ret_T dkim_readkey(dkim_s_P _dkim_s);
sm_ret_T dkim_msg_free(dkim_msg_P _dkim_msg);
sm_ret_T dkim_msg_new(dkim_ctx_P _dkim_ctx, dkim_msg_P *_pdkim_msg);
sm_ret_T dkim_msg_open(dkim_ctx_P _dkim_ctx, dkim_msg_P *_pdkim_msg);
sm_ret_T dkim_msg_close(dkim_msg_P _dkim_msg);

sm_ret_T dkim_msg_chunk(dkim_msg_P _dkim_msg, const uchar *_msg, size_t _len);

sm_ret_T dkim_conf_init(dkim_ctx_P _dkim_ctx, sm_maps_P *_pmaps_ctx);

#endif /* MTA_USE_DKIM */

#endif /* SM_DKIM_H */
