/*
 * Copyright (c) 2010, 2011 Sendmail, Inc. and its suppliers.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 *	$Id: dkimcnf.h,v 1.20 2013/12/13 02:08:19 ca Exp $
 */

#ifndef SM_DKIMCNF_H
#define SM_DKIMCNF_H 1

#include "sm/generic.h"
#include "sm/magic.h"

#if MTA_USE_DKIM

#define DKIM_HDR_LIST_MAX	64

/* check for this? */
#include <openssl/ssl.h>
#include <openssl/bio.h>
#include <openssl/pem.h>

/*
 *  DKIM selector
 */

typedef struct dkim_s_S	dkim_s_T, *dkim_s_P;
struct dkim_s_S
{
	/* configuration data */
	char		*dks_name;	/* optional name of this conf. section */
	char		*dks_keyfile;	/* file with private key */
	char		*dks_selector;
	char		*dks_domain;
	char		*dks_from;	/* restrict to From: address */

	/* key data */
	uchar		*dks_priv_key;
	size_t		 dks_priv_key_len;

	BIO			*dks_bio_key;
	EVP_PKEY	*dks_rsa_pkey;
	RSA			*dks_rsa;
	uchar		 dks_rsa_pad;
	size_t		 dks_rsa_keysize;
	uchar		*dks_rsaout;
};

/*
 *  DKIM configuration ("global")
 */


typedef struct dkim_cnf_S	dkim_cnf_T, *dkim_cnf_P;
struct dkim_cnf_S
{
	sm_magic_T	 sm_magic;
	uint		 dkimcnf_signmethod;
	uint		 dkimcnf_bcanon;	/* body canon. */
	uint		 dkimcnf_hcanon;	/* header canon. */
	bool		 dkimcnf_ignerr;	/* ignore errors? */

	char		*dkimcnf_sign[DKIM_HDR_LIST_MAX];
	char		*dkimcnf_skip[DKIM_HDR_LIST_MAX];
	bool		 dkimcnf_no_sign;	/* was _sign empty? */
	bool		 dkimcnf_no_skip;	/* was _skip empty? */
	char		*dkimcnf_add[DKIM_HDR_LIST_MAX];

	dkim_s_T	*dkimcnf_s;
	uint		 dkimcnf_n;

	uint32_t	 dkimcnf_hdrtlen;
	uint32_t	 dkimcnf_hdrvlen;
};

#define DKIM_IGNERR(dkim_cnf)	((dkim_cnf)->dkimcnf_ignerr)
#define DKIM_SIGNM(dkim_cnf) ((dkim_cnf)->dkimcnf_signmethod)

#endif /* MTA_USE_DKIM */

#endif /* SM_DKIMCNF_H */
