/*
 * Copyright (c) 2002-2005 Sendmail, Inc. and its suppliers.
 *      All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 *	$Id: net.h,v 1.50 2013/09/25 21:59:09 ca Exp $
 */

#ifndef SM_NET_H
#define SM_NET_H 1

#include "sm/generic.h"
#include "sm/str.h"

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/un.h>

#if MTA_NETINET6 && !HAVE_INET_PTON
# ERROR: _MTA_NETINET6 set but inet_pton is missing.
#endif

#ifndef INET_ADDRSTRLEN
# define INET_ADDRSTRLEN	16
#endif
#ifndef INET_ARPASTRLEN
# define INET_ARPASTRLEN	40
#endif

#if MTA_NETINET6
# ifndef INET6_ADDRSTRLEN
#  define INET6_ADDRSTRLEN	46
# endif
# ifndef INET6_ARPASTRLEN
#  define INET6_ARPASTRLEN	74
# endif
# define IP_ADDRSTRLEN	INET6_ADDRSTRLEN
# define IP_ARPASTRLEN	INET6_ARPASTRLEN
#else
# define IP_ADDRSTRLEN	INET_ADDRSTRLEN
# define IP_ARPASTRLEN	INET_ARPASTRLEN
#endif
#define IP_ADDRSTRMAX	(IP_ADDRSTRLEN + 4)
#define IP_ARPASTRMAX	(IP_ARPASTRLEN + 4)

#define NETUNIX	1
#define NETINET	1
#define NETINET6 MTA_NETINET6

#ifndef INADDR_NONE
# define INADDR_NONE	0xffffffff
#endif

#ifndef INADDR_LOOPBACK
# define INADDR_LOOPBACK	0x7f000001
#endif

typedef struct in_addr		in_addr_T, *in_addr_P;
/* Based on KAME, *BSD */
struct sm_in6_addr {
	union {
		uint8_t   sm6_addr8[16];
		uint16_t  sm6_addr16[8];
		uint32_t  sm6_addr32[4];
	} sm6_addr;
};
typedef struct sm_in6_addr		in6_addr_T, *in6_addr_P;

/* XXX This should be in_addr */
typedef uint32_t ipv4_T;
typedef in6_addr_T ipv6_T, *ipv6_P;

/* compare two IPv4 addresses; need later on generic address comparison macro */
#define ipv4_addr_eq(a1, a2)	((a1) == (a2))
#define ipv6_addr_eq(a1, a2)	sm_memeq((a1), (a2), sizeof(ipv6_T))
#define IPV6_ASSIGN(d, s)	sm_memcpy((d), (s), sizeof(ipv6_T))
#define SOCKADDR_ASSIGN(d, s)	sm_memcpy((d), (s), sizeof(sm_sockaddr_T))

#define sockaddr_len_T socklen_T

/* generic Unix sockaddr structure */
typedef struct sockaddr		sockaddr_T, *sockaddr_P;

/* Unix sockaddr structure for IPv4 */
typedef struct sockaddr_in	sockaddr_in_T, *sockaddr_in_P;
#if MTA_NETINET6
typedef struct sockaddr_in6	sockaddr_in6_T, *sockaddr_in6_P;
#endif

#if MTA_NETINET6
# define GET_SOCKADDR_PORT(sockaddr)				\
		(((sockaddr).sa.sa_family == AF_INET6) ?	\
			ntohs((sockaddr).sin6.sin6_port) : 		\
			((sockaddr).sa.sa_family == AF_INET) ?	\
				ntohs((sockaddr).sin.sin_port): -1)
# define SET_SOCKADDR_PORT(sockaddr, port) do {		\
		if ((sockaddr).sa.sa_family == AF_INET6)	\
			(sockaddr).sin6.sin6_port = (port);	\
		else if ((sockaddr).sa.sa_family == AF_INET)	\
				(sockaddr).sin.sin_port = (port);	\
	} while (0)

# define GET_SOCKADDR_LEN(sockaddr)					\
		(((sockaddr).sa.sa_family == AF_INET6) ?	\
		sizeof((sockaddr).sin6) : 					\
		((sockaddr).sa.sa_family == AF_INET) ?		\
		sizeof((sockaddr).sin):						\
		((sockaddr).sa.sa_family == AF_UNIX) ?		\
		sizeof((sockaddr).sunix): -1)

#else
# define GET_SOCKADDR_PORT(sockaddr)			\
		(((sockaddr).sa.sa_family == AF_INET) ?	\
			ntohs((sockaddr).sin.sin_port): -1)
# define SET_SOCKADDR_PORT(sockaddr, port) (sockaddr).sin.sin_port = (port)

# define GET_SOCKADDR_LEN(sockaddr)					\
		(((sockaddr).sa.sa_family == AF_INET) ?		\
		sizeof((sockaddr).sin):						\
		((sockaddr).sa.sa_family == AF_UNIX) ?		\
		sizeof((sockaddr).sunix): -1)
#endif

/*
**  generic sockaddr structure for sendmail
**  make parts of this conditional, compare sendmail8/sendmail/sendmail.h
*/

union sm_sockaddr_U
{
	struct sockaddr		sa;	/* general version */
	struct sockaddr_un	sunix;	/* UNIX family */
	struct sockaddr_in	sin;	/* INET family */
#if MTA_NETINET6
	struct sockaddr_in6	sin6;	/* INET6 family */
#endif
};

typedef union sm_sockaddr_U	sm_sockaddr_T, *sm_sockaddr_P;

union sm_ip_addr_U
{
	ipv4_T	ipv4_addr;
#if MTA_NETINET6
	ipv6_T	ipv6_addr;
#endif
};
typedef union sm_ip_addr_U	sm_ip_addr_T, *sm_ip_addr_P;

struct sm_ipaddr_S
{
	int				ipa_family;
	sm_ip_addr_T	ipa_addr;
};

typedef struct sm_ipaddr_S	sm_ipaddr_T, *sm_ipaddr_P;

/* XXX */
#define closesocket close

#define INVALID_SOCKET (-1)
#define is_valid_socket(fd)	((fd) >= 0)
#define CLOSE_SOCKET(fd)	do {			\
			if (is_valid_socket(fd)) {		\
				close(fd);		\
				(fd) = INVALID_SOCKET;	\
			}				\
		} while (0)

#if MTA_NETINET6
# define IP_SIZE(af)	\
	((af) == AF_INET  ? sizeof(ipv4_T):\
	 (af) == AF_INET6 ? sizeof(ipv6_T):\
	0)

# define VALID_IP(ipaddr)	\
	(((ipaddr).ipa_family) == AF_INET || ((ipaddr).ipa_family) == AF_INET6)

# define GETIPA(ipaddr)	\
	((((ipaddr).ipa_family) == AF_INET) ?	\
	 (uchar *)&(ipaddr).ipa_addr.ipv4_addr:\
	 (uchar *)&(ipaddr).ipa_addr.ipv6_addr)
# define IPASIZE(ipaddr)	\
	(((ipaddr).ipa_family) == AF_INET  ? sizeof(ipv4_T):\
	 ((ipaddr).ipa_family) == AF_INET6 ? sizeof(ipv6_T):\
	0)

# define GETIP(sckaddr)	\
	(((sckaddr).sa.sa_family) == AF_INET  ? (uchar *)&(sckaddr).sin.sin_addr:\
	 (uchar *)&(sckaddr).sin6.sin6_addr)
# define IPSIZE(sckaddr)	\
	(((sckaddr).sa.sa_family) == AF_INET  ? sizeof((sckaddr).sin.sin_addr):\
	 ((sckaddr).sa.sa_family) == AF_INET6 ? sizeof((sckaddr).sin6.sin6_addr):\
	0)
# define SOCKSIZE(sckaddr)	\
	(((sckaddr).sa.sa_family) == AF_INET  ? sizeof((sckaddr).sin):\
	 ((sckaddr).sa.sa_family) == AF_INET6 ? sizeof((sckaddr).sin6):\
	 ((sckaddr).sa.sa_family) == AF_UNIX  ? sizeof((sckaddr).sunix):\
	0)

# define SCK_ASSIGN_IP(sckaddr, ipaddr) do { \
		(sckaddr).sa.sa_family = (ipaddr).ipa_family;	\
		sm_memcpy(GETIP(sckaddr), GETIPA(ipaddr), sizeof(ipv6_T));	\
	} while (0)

# define IP_ASSIGN_SCK(ipaddr, sckaddr) do { \
		(ipaddr).ipa_family = (sckaddr).sa.sa_family;	\
		sm_memcpy(GETIPA(ipaddr), GETIP(sckaddr), sizeof(ipv6_T));	\
	} while (0)

/*
 * provide key and length for HT lookups.
 * full IP address as key (including family!)
 */
# define HT_IPA_KEYaLEN(ipaddr) \
	(void *)&(ipaddr),  \
	((((ipaddr).ipa_family) == AF_INET  ? sizeof(ipv4_T):\
	 ((ipaddr).ipa_family) == AF_INET6 ? sizeof(ipv6_T):\
	0)+sizeof((ipaddr).ipa_family))

#define IP_ASSIGN_IPV6(ipaddr, ipv6) do { \
		(ipaddr).ipa_family = AF_INET6;	\
		(ipaddr).ipa_addr.ipv6_addr = (ipv6);	\
	} while (0)

#else /* MTA_NETINET6 */

# define IP_SIZE(af)	sizeof(ipv4_T)
# define VALID_IP(ipaddr)	(((ipaddr).ipa_family) == AF_INET)

# define GETIPA(ipaddr)	((uchar *)&(ipaddr).ipa_addr.ipv4_addr)
# define IPASIZE(ipaddr)	sizeof(ipv4_T)

# define GETIP(sckaddr)	((uchar *)&(sckaddr).sin.sin_addr.s_addr)
# define IPSIZE(sckaddr)	\
	(((sckaddr).sa.sa_family) == AF_INET  ? sizeof((sckaddr).sin.sin_addr):\
	0)
# define SOCKSIZE(sckaddr)	\
	(((sckaddr).sa.sa_family) == AF_INET  ? sizeof((sckaddr).sin):\
	 ((sckaddr).sa.sa_family) == AF_UNIX  ? sizeof((sckaddr).sunix):\
	0)

# define SCK_ASSIGN_IP(sckaddr, ipaddr) do { \
		(sckaddr).sa.sa_family = (ipaddr).ipa_family;	\
		(sckaddr).sin.sin_addr.s_addr = (ipaddr).ipa_addr.ipv4_addr;	\
	} while (0)

# define IP_ASSIGN_SCK(ipaddr, sckaddr) do { \
		(ipaddr).ipa_family = (sckaddr).sa.sa_family;	\
		(ipaddr).ipa_addr.ipv4_addr = (sckaddr).sin.sin_addr.s_addr;	\
	} while (0)

/*
 * provide key and length for HT lookups.
 * full IP address as key (including family!)
 */
# define HT_IPA_KEYaLEN(ipaddr) \
	(void *)&(ipaddr), (sizeof(ipv4_T)+sizeof((ipaddr).ipa_family))

#endif /* MTA_NETINET6 */

#define IP_A_ASSIGN_IPA(ip_a, family, ipa)	(ip_a) = (ipa).ipa_addr
#define IPA_ASSIGN_IP_A(ipa, family, ip_a)	do {	\
	(ipa).ipa_addr = (ip_a);	\
	(ipa).ipa_family = (family);	\
	} while (0)

#define IPADDR_ASSIGN(d, s)	sm_memcpy((d), (s), sizeof(sm_ipaddr_T))
#define IPADDR_ZERO(d)	sm_memzero((d), sizeof(*(d)))
#define IPADDR_EQ(a1, a2)	((a1).ipa_family == (a2).ipa_family && \
		    sm_memeq(GETIPA(a1), GETIPA(a2), IPASIZE(a1)))

#define IN_IS_ADDR_LOOPBACK(addr)	((addr).sin.sin_addr.s_addr == ntohl(INADDR_LOOPBACK))

#define IP_ASSIGN_IPV4(ipaddr, ipv4) do { \
		(ipaddr).ipa_family = AF_INET;	\
		(ipaddr).ipa_addr.ipv4_addr = (ipv4);	\
	} while (0)

#define IPV4_ASSIGN_IP(ipv4, ipaddr) do { \
		SM_ASSERT(AF_INET == (ipaddr).ipa_family);	\
		(ipv4) = (ipaddr).ipa_addr.ipv4_addr;	\
	} while (0)

#define IPV4_GET_IP(ipv4, ipaddr) do { \
		if (AF_INET == (ipaddr).ipa_family)	\
			(ipv4) = (ipaddr).ipa_addr.ipv4_addr;	\
		else \
			(ipv4) = INADDR_NONE;	\
	} while (0)

#if MTA_NETINET6
# define IP_IS_ADDR_LOOPBACK(addr)	\
	(((addr).sa.sa_family == AF_INET) ? IN_IS_ADDR_LOOPBACK(addr) : \
	 ((addr).sa.sa_family == AF_INET6) ? IN6_IS_ADDR_LOOPBACK(&((addr).sin6.sin6_addr)) : false)
#else
# define IP_IS_ADDR_LOOPBACK(addr)	IN_IS_ADDR_LOOPBACK(addr)
#endif

sm_ret_T net_client_connect(const char *_ip, int _port, int *pfd);
sm_ret_T net_client_connectipv4(ipv4_T ipv4, int port, int *pfd);
int	net_server_listen_addr(sockaddr_T *_addr, int _addrlen, int _backlog);
sm_ret_T net_server_listen(const char *_ip, int _port, int _backlog);
sm_ret_T net_server_accept(int _listenfd, sockaddr_T *_addr, sockaddr_len_T *_addrlen);

sm_ret_T sm_inet_a2ipv4(const char *_cp, char **_endptr, ipv4_T *_ipv4);
sm_ret_T sm_inet_ipv4str(ipv4_T _ipv4, sm_str_P _dst);
sm_ret_T sm_inet_inaddr2str(in_addr_T _inaddr, sm_str_P _dst);
sm_ret_T sm_inet_ipv42arpastr(ipv4_T _ipv4, sm_str_P _dst);
sm_ret_T sm_inet_inaddr2arpastr(struct in_addr _inaddr, sm_str_P _dst);
sm_ret_T sm_inet_arpa2ipv4(const char *_cp, ipv4_T *_ipv4);
sm_ret_T sm_inet_a2sockaddr(const char *_cp, char **_end, sm_sockaddr_P _sockaddr);
sm_ret_T sm_inet_a2ip(const char *_cp, char **_end, sm_ipaddr_P _sockaddr);

#if MTA_NETINET6
sm_ret_T sm_inet_a2ipv6(const char *_src, char **_end, ipv6_P _ipv6);
sm_ret_T sm_inet_in6addr2arpastr(struct in6_addr _in6addr, sm_str_P _dst);
sm_ret_T sm_inet_in6addr2str(struct in6_addr _in6addr, sm_str_P _dst);
sm_ret_T sm_inet_ipv62arpastr(ipv6_P _ipv6, sm_str_P _dst);
sm_ret_T sm_inet_ipv6str(ipv6_P _ipv6, sm_str_P _dst);
sm_ret_T sm_inet_ipv6fstr(ipv6_P _ipv6, sm_str_P _dst);
sm_ret_T sm_inet_in6faddrstr(struct in6_addr _in6addr, sm_str_P _dst);
#endif

sm_ret_T sm_sockaddr2str(sm_sockaddr_P _addr, sm_str_P _dst);
sm_ret_T sm_ipaddr2str(sm_ipaddr_P _ipaddr, sm_str_P _dst);

sm_ret_T sm_inet_ip2arpastr(sm_ipaddr_P _ipaddr, sm_str_P _dst);

#endif /* SM_NET_H */
