#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-backslash.c,v 1.3 2010/06/19 14:47:06 ca Exp $")
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/string.h"
#include "sm/io.h"
#include "sm-conf-token.h"
#include "sm/test.h"

static int Verbosity = 0;

#define LEN	256

struct tr_S {
	char *e_string;
	char *e_result;
	int   e_err;
};
typedef struct tr_S tr_T;

static tr_T trs[] = {
	{ "\"abc\"",		"abc",			0},
	{ "\"c\"",			"c",			0},
	{ "\"\\\\\"",		"\\\\",			0},
	{ "\"\\007\"",		"\007",			0},
	{ "\"\\a\"",		"\a",			0},
	{ "\"a\\ab\"",		"a\ab",			0},
	{ "\"\\",			"\\",			SM_CONF_ERR_EOF_IN_STRING},
	{ "\"a\\.c\"",		"a\\.c",		0},
	{ "\"\\.\"",		"\\.",		0},
	{ NULL, NULL, 0}
};

#define STR_QUOTE	'"'

static int
lexstring(char *str, char *endptr, char **rest, size_t *plen)
{
	char *rptr, *wptr;

	SM_REQUIRE(str != NULL);
	SM_REQUIRE(endptr != NULL);
	SM_REQUIRE(*str == STR_QUOTE);

	++str;
	for (rptr = wptr = (char *)str; rptr < endptr; ) {
		if (*rptr == '\\') {
			int err;

			++rptr;
			err = sm_conf_backslash(&wptr, &rptr, endptr);
			if (err != 0)
				return err;
			continue;
		}
		if (*rptr == STR_QUOTE)
			break;
		*wptr++ = *rptr++;
	}
	if (*rptr != STR_QUOTE)
		return SM_CONF_ERR_EOF_IN_STRING;
	*rptr = '\0';
	++rptr;
	if (rest != NULL)
		*rest = rptr;
	if (plen != NULL)
		*plen = wptr - str;
	return 0;
}

static void
test(void)
{
	int i, err;
	size_t l;
	char *s, *r, *e;
	char buf[1024];

	for (i = 0; trs[i].e_string != NULL; i++) {
		r = trs[i].e_string;
		err = strlcpy(buf, r, sizeof(buf));
		s = buf;
		l = strlen(s);
		e = buf + l;
		err = lexstring(s, e, &r, &l);
		SM_TEST(err == trs[i].e_err);
		if (err == 0) {
			++s;
			SM_TEST(strncmp(s, trs[i].e_result, l) == 0);
			if (Verbosity > 0 && strncmp(s, trs[i].e_result, l) != 0) {
				fprintf(stderr, "i=%d, s=%*s, r=%*s\n", i, (int) l, s, (int) l, trs[i].e_result);
			}
			SM_TEST(l == strlen(trs[i].e_result));
			if (Verbosity > 0 && l != strlen(trs[i].e_result)) {
				fprintf(stderr, "i=%d, l=%lu, len=%lu\n", i, (ulong) l, (ulong) strlen(trs[i].e_result));
			}
			SM_TEST(l == strlen(trs[i].e_result));
		}
	}
	return;
}

int
main(int argc, char *argv[])
{
	int c;
	bool enhsc;

	enhsc = false;
	while ((c = getopt(argc, argv, "V")) != -1) {
		switch (c) {
		  case 'V':
			++Verbosity;
			break;
		}
	}
	sm_test_begin(argc, argv, "test backslash");

	argc -= optind;
	argv += optind;

	test();

	return sm_test_end();
}
