#!/bin/sh
# $Id: t-dkim-0.sh,v 1.8 2010/12/22 19:46:49 ca Exp $
# Copyright (c) 2010 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test dkim signing "by hand" (hand crafted test program)
# ----------------------------------------

test -s stop && exit 1              

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../libdkim
fi
ERRS=0
PRG=./t-dkim-0
BODY=msg
OUT=t-dkim-0.out

if $PRG -C | grep MTA_USE_DKIM >/dev/null
then
  :
else
   echo "SKIPPED: $0: $PRG not compiled with MTA_USE_DKIM"
   exit 0
fi

# ----------------------------------------
TEST=1
ERR=false
cat >$BODY <<EOF
2020 Vision Street
.Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
.email: mmtest3@v253.pogonip.net
EOF
$PRG -h '05duZagydiBtdrpaG7UQeScqnfMuIpoVQXkGEtWHmKg=' -f $BODY || ERR=true
$ERR && ERRS=`expr ${ERRS} + 1 `

# ----------------------------------------
TEST=2
ERR=false
cat >$BODY <<EOF
2020 Vision Street
..Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
..email: mmtest3@v253.pogonip.net
EOF
$PRG -p -h '05duZagydiBtdrpaG7UQeScqnfMuIpoVQXkGEtWHmKg=' -f $BODY || ERR=true
$ERR && ERRS=`expr ${ERRS} + 1 `

# ----------------------------------------
TEST=3
ERR=false
cat >$BODY <<EOF
From: user <user@my.domain>
To: rcpt <rcpt@your.domain>
Subject: whatever
Date: Thu, 05 May 2005 11:59:09 -0700
Message-ID: <439094BF.5010709@sendmail.com>

2020 Vision Street
..Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
..email: mmtest3@v253.pogonip.net
EOF
$PRG -h '05duZagydiBtdrpaG7UQeScqnfMuIpoVQXkGEtWHmKg=' -m $BODY || ERR=true
$ERR && ERRS=`expr ${ERRS} + 1 `

# ----------------------------------------
TEST=4
ERR=false
cat >$BODY <<EOF
From: user@example.com
To: recipient@sendmail.net
Subject: test DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
EOF
$PRG -h 'P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E=' -m $BODY || ERR=true
$ERR && ERRS=`expr ${ERRS} + 1 `

# ----------------------------------------
TEST=5
ERR=false
cat >$BODY <<EOF
From: user@example.com
To: recipient@sendmail.net
Subject: test DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
EOF
H="P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E="
$PRG -k $SD/private.key -m $BODY -h "$H" -VV -s "$@" >$OUT 2>&1 || ERR=true
$ERR && ERRS=`expr ${ERRS} + 1 `

SIG="LMogMTvrbyx4VeI4Yqw5K17rdFBTuMmuFGWKfJDxFHuVirOoOAA9SYDFucEYQVhOdgwqlVbJszGxQw0PMNqvwdWYRmQ5JKV7Uq+3oRImRX+zmGr5ZiHQxYXYSV5T5m0SQx3HQjB3qM3DiVHVAovhBgqDFfHqglC0jxVfITMQP2w="
if fgrep "${SIG}" ${OUT} >/dev/null
then
  :
else
  echo "$0: ${TEST}: signature $SIG not found in ${OUT}"
  ERRS=`expr ${ERRS} + 1 `
fi


if test ${ERRS} -eq 0
then
  rm -f ${BODY} ${OUT}
  exit 0
else
  echo "${ERRS} error(s)"
  exit 1
fi
