#!/bin/sh
# $Id: t-dkim-conf-3.sh,v 1.5 2013/12/13 02:31:41 ca Exp $
# Copyright (c) 2011 Sendmail, Inc. and its suppliers.
#	All rights reserved.
#
# By using this file, you agree to the terms and conditions set
# forth in the LICENSE file which can be found at the top level of
# the sendmail distribution.
#
# ----------------------------------------
# test dkim signing, using a configuration file
# ----------------------------------------

test -s stop && exit 1              

SD=`dirname $0`
if test "${SD}" = "."
then
   SD=../libdkim
fi
. ${SD}/common.sh

# ----------------------------------------
# duplicate headers
TEST=1
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
H="P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E="
SIG="ZJa7YmqhDWE1vYoXzg3skbeawy4XquuYavOSxTmKaCZ+aOLofr70cFHEKfkDCoxm8CKy3BuWavvbzYl5+nbIYrnkVnk2W3YAZ/L3dBHiYur1go1910s/1A9/cvyOzzJkbkA+25BaO0DJVbYkDiN3JYRRRDbhM+jZkSQ/d5Fm3WQ="

cat >$BODY <<EOF
From: user@example.com
To: other@example.net
To: recipient@sendmail.net
Subject: test DKIM

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
.
EOF

cat >$CONF <<EOF
sign {
   domain=example.net;
   selector=default;
   keyfile=${K}/private.key;
   from=example.com;
}
sign net {
   domain=example.net;
   selector=net;
   keyfile=${K}/net.key;
}
EOF

$PRG -t ${DKTIME} -f $CONF -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi

# ----------------------------------------
# duplicate headers (3x)
TEST=2
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then
H="P7M5AsnZKX0CSpQl3w3u9XwzYQ7gDrSE5gSSQy/1W2E="
SIG="WluFuiNWLoh2PSkXsU5jge1Ep5tBpi/q+hNIZST41I9i/E0Enb1va8xoWsAk6kgCpgBI2eiqrWrQEx9WXvNXmpzQxg6+JN1erGNtbK7oRKmf5vcrF0u8sag2RhdDnCgCdo8n+0sGKafdS1EP6xPxd6e7GTlFV3+cLLfiyaLqjWI="


cat >$BODY <<EOF
To: other@example.net
From: user@example.com
To: recipient@sendmail.net
Subject: test DKIM
To: third@sendmail.net

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
.
EOF

cat >$CONF <<EOF
sign {
   domain=example.net;
   selector=default;
   keyfile=${K}/private.key;
}
EOF

$PRG -t ${DKTIME} -f $CONF -m $BODY -h "bh=$H" -S "b=${SIG}" -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# ----------------------------------------
# headers too large
TEST=3
ERR=false
if echo " ${SKIP} X" | grep -v " ${TEST} " > /dev/null
then

cat >$BODY <<EOF
From: user@example.com
To: recipient@sendmail.net
Subject: test DKIM
HEADERNAME: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME1: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME2: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME3: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME4: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME5: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME6: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME7: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME8: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME9: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME10: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME11: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME12: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME13: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME14: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME15: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME16: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME17: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME18: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME19: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME20: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME21: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME22: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME23: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME24: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest
HEADERNAME25: testtesttesttesttesttesttesttesttesttesttesttesttesttesttesttest

2020 Vision Street
Anycity, Anystate 90009
Office ph: (510) 555-1234
Cell ph:   (510) 555-5678
fax:       (510) 555-9000
email: mmtest3@v253.pogonip.net
.
EOF

cat >$CONF <<EOF
sign {
   domain=example.net;
   selector=default;
   keyfile=${K}/private.key;
}
EOF

$PRG -t ${DKTIME} -f $CONF -m $BODY -e 0x94040106 -VV "$@" >$OUT 2>&1 || ERR=true
if $ERR
then
  ERRS=`expr ${ERRS} + 1 `
  ${VERBOSE} && echo "$0: test ${TEST} failed"
  test X"${MTA_STOPONERROR}" != X && exit 1
fi
else
  ${VERBOSE} && echo "$0: test ${TEST} SKIPPED"
fi


# ----------------------------------------
if test ${ERRS} -eq 0
then
  rm -f ${BODY} ${OUT} ${CONF}
  exit 0
else
  echo "${ERRS} error(s)"
  exit 1
fi
