#include "sm/generic.h"
SM_RCSID("@(#)$Id: sm_inet_a2ip.c,v 1.3 2013/12/03 00:21:17 ca Exp $")

#include "sm/assert.h"
#include "sm/error.h"
#include "sm/memops.h"
#include "sm/net.h"

/*
**  SM_INET_A2IP -- (check and) convert ASCII version of IP address to binary
**
**	Parameters:
**		cp -- begin of ASCII representation
**		end -- (pointer to) end of ASCII representation (output)
**		ipaddr -- ip addr (output)
**
**	Returns:
**		usual sm_error code; ERANGE, EINVAL, ...
*/

sm_ret_T
sm_inet_a2ip(const char *cp, char **end, sm_ipaddr_P ipaddr)
{
	sm_ret_T ret;

	SM_REQUIRE(cp != NULL);
	SM_REQUIRE(ipaddr != NULL);

	sm_memzero(ipaddr, sizeof(*ipaddr));
	ret = sm_inet_a2ipv4(cp, end, &(ipaddr->ipa_addr.ipv4_addr));
	if (sm_is_ok(ret)) {
		ipaddr->ipa_family = AF_INET;
		return ret;
	}
#if MTA_NETINET6
	ret = sm_inet_a2ipv6(cp, end, &(ipaddr->ipa_addr.ipv6_addr));
	if (sm_is_ok(ret)) {
		ipaddr->ipa_family = AF_INET6;
		return ret;
	}
#endif
	return sm_error_perm(SM_EM_IP, EINVAL);
}
