/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: mtparse.c,v 1.17 2014/01/30 16:02:28 ca Exp $")
#include "sm/common.h"
#include "sm/error.h"
#include "sm/io.h"
#include "smar.h"
#include "log.h"
#include "sm/sm-conf.h"
#include "sm/sm-conf-prt.h"
#include "sm/smarcnf.h"
#include "sm/smarcnfdef.h"
#include "mtparse.h"

/*
RHS      ::= "{" ["protocol" "=" "esmtp" ";"] [port] hosts [flags] "}" |
		     "{" "protocol" "=" "lmtp" ";" [port hosts] "}"
port     ::= "port" "=" int ";"
flags    ::= "flags" "=" "MX_lookup" ";"
protocol ::= "protocol" "=" ( "lmtp" | "esmtp") ";"
hosts    ::= "host" "=" hostname  | ips
ips      ::= "ip" "=" "{" ip-list "}"
         ::= ("dest" "{" "ip" "=" ip ";" "pref" "=" uint ";" "}")*
ip      ::= IP-address
*/

static sm_conf_definition_T const
ar_mt_flag_names[] = {
/*
{ SM_CONF_DEF_MAGIC,
	"A_lookup",		sm_conf_type_choice_value,
	SMAR_MT_CNF_A_ONLY,
	0, NULL, 0, NULL, NULL, NULL,
	"perform only A lookup"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
*/

{ SM_CONF_DEF_MAGIC,
	"MX_lookup",		sm_conf_type_choice_value,
	SMAR_MT_CNF_MX,
	0, NULL, 0, NULL, NULL, NULL,
	"perform MX lookup"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

CONF_SENTINEL
};

static sm_conf_definition_T const
ar_prot_names[] = {
{ SM_CONF_DEF_MAGIC, "esmtp",		sm_conf_type_choice_value,
	SMAR_PROT_ESMTP,
	0, NULL, 0, NULL, NULL, NULL,
	"ESMTP"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },
{ SM_CONF_DEF_MAGIC, "lmtp",	sm_conf_type_choice_value,
	SMAR_PROT_LMTP,
	0, NULL, 0, NULL, NULL, NULL,
	"LMTP"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },


/* Sentinel */
{ SM_CONF_DEF_MAGIC, NULL, 0, 0, 0, NULL, 0, NULL, NULL, NULL, NULL SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0)}
};

sm_conf_definition_T mt_ip_def[] = {
{ SM_CONF_DEF_MAGIC, "ip", sm_conf_type_ip,
	offsetof(ar_mt_ipp_T, ipp_addr), sizeof(sm_ipaddr_T),
	NULL, 0,
	NULL, NULL,	NULL,
	"IP"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "pref",		sm_conf_type_u32,
	offsetof(ar_mt_ipp_T, ipp_pref),	sizeof(uint32_t),
	NULL, 0,
	NULL, NULL,	NULL,
	"pref"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

	CONF_SENTINEL
};

sm_conf_definition_T mt_ip_definitions[] = {
{ SM_CONF_DEF_MAGIC, "",	sm_conf_type_section,
	offsetof(ar_mtcnf_T, ar_mt_cnf_ipp), sizeof(ar_mt_ipp_T),
	NULL, SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_KEEP_DEFAULT,
	mt_ip_def,
	NULL, NULL, "ips" },

{ SM_CONF_DEF_MAGIC, "n",	sm_conf_type_array_n,
	offsetof(ar_mtcnf_T, ar_mt_cnf_n_ips), sizeof(uint),
	NULL, 0,
	mt_ip_def,
	NULL, NULL,
	"number of ips (implicit)" },

CONF_SENTINEL
};

static sm_conf_definition_T
ar_mtrhs_defs[] = {
{ SM_CONF_DEF_MAGIC, "protocol",		sm_conf_type_choice,
	offsetof(ar_mtcnf_T, ar_mt_cnf_prot),	sizeof(uint),
	NULL, 0,
	ar_prot_names,	NULL,	NULL,
	"protocol name"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "port",		sm_conf_type_u32,
	offsetof(ar_mtcnf_T, ar_mt_cnf_port),	sizeof(ushort),
	NULL,
	0,	NULL,	NULL,	NULL,
	"port number"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "ip",		sm_conf_type_argv,
	offsetof(ar_mtcnf_T, ar_mt_cnf_ips),	SMAR_MT_IP_MAX,
	NULL,	0,	NULL,	NULL,	NULL,
	"List of hosts (IP addresses)"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

/* backwards compatibility -- remove after 2 releases! */
{ SM_CONF_DEF_MAGIC, "ipv4",		sm_conf_type_argv,
	offsetof(ar_mtcnf_T, ar_mt_cnf_ips),	SMAR_MT_IP_MAX,
	NULL,	0,	NULL,	NULL,	NULL,
	"List of hosts (IP addresses)"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "host",		sm_conf_type_string,
	offsetof(ar_mtcnf_T, ar_mt_cnf_host),	0,
	NULL,	0,	NULL,	NULL,	NULL,
	"hostname"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "flags",		sm_conf_type_choice,
	offsetof(ar_mtcnf_T, ar_mt_cnf_flags),		sizeof(uint),
	"MX_lookup", SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_OR|SM_CONF_FLAG_PLURAL,
	ar_mt_flag_names, NULL,	NULL,
	"flags to influence behaviour"
	SM_LC_NO_ISSET SM_LC_SET_MAGIC(0) },

{ SM_CONF_DEF_MAGIC, "dest",		sm_conf_type_array,
	0,	/* offset */
	SMAR_MT_IP_MAX,	/* size (# of elements) */
	NULL,	/* default */
	SM_CONF_FLAG_MULTIPLE|SM_CONF_FLAG_KEEP_DEFAULT|SM_CONF_FLAG_FLAT,
	mt_ip_definitions,
	NULL,  NULL,
	"ip declarations"
	SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0) },

/* Sentinel */
{ SM_CONF_DEF_MAGIC, NULL, 0, 0, 0, NULL, 0, NULL, NULL, NULL, NULL SM_LC_NO_ISSET	SM_LC_SET_MAGIC(0)}
};

sm_ret_T
smar_mtcnf_clear(ar_mtcnf_P ar_mtcnf)
{
	if (NULL == ar_mtcnf)
		return SM_SUCCESS;
	if (ar_mtcnf->ar_mt_cnf_smc != NULL) {
		sm_conf_destroy(ar_mtcnf->ar_mt_cnf_smc);
		ar_mtcnf->ar_mt_cnf_smc = NULL;
	}
	sm_memzero(ar_mtcnf, sizeof(*ar_mtcnf));
	return SM_SUCCESS;
}

static sm_ret_T
smar_mtipp_sort(ar_mtcnf_P ar_mtcnf)
{
	int i, j;
	int min_idx;
	uint32_t min_pref;

	if (NULL == ar_mtcnf)
		return SM_SUCCESS;

	/* this is slow, but SMAR_MT_IP_MAX is very small */
	for (i = 0; i < ar_mtcnf->ar_mt_cnf_n_ips; i++) {
		min_idx = i;
		min_pref = ar_mtcnf->ar_mt_cnf_ipp[i].ipp_pref;
		for (j = i + 1; j < ar_mtcnf->ar_mt_cnf_n_ips; j++) {
			if (ar_mtcnf->ar_mt_cnf_ipp[j].ipp_pref < min_pref) {
				min_idx = j;
				min_pref = ar_mtcnf->ar_mt_cnf_ipp[j].ipp_pref;
			}
		}
		if (min_idx != i) {
			char *ips;
			ar_mt_ipp_T ipp;

			ipp = ar_mtcnf->ar_mt_cnf_ipp[i];
			ar_mtcnf->ar_mt_cnf_ipp[i] = ar_mtcnf->ar_mt_cnf_ipp[min_idx];
			ar_mtcnf->ar_mt_cnf_ipp[min_idx] = ipp;
			ips = ar_mtcnf->ar_mt_cnf_ips[i];
			ar_mtcnf->ar_mt_cnf_ips[i] = ar_mtcnf->ar_mt_cnf_ips[min_idx];
			ar_mtcnf->ar_mt_cnf_ips[min_idx] = ips;
		}
	}
	return SM_SUCCESS;
}

/*
**  SMAR_MT_PARSE -- Parse RHS of mt entry
**
**	Parameters:
**		smar_clt_ctx -- SMAR client context
**		mtrhs -- RHS from mailertable
**		ar_mtcnf -- result structure containing parsed data etc
**
**	Returns:
**		usual sm_error code
*/

sm_ret_T
smar_mt_parse(smar_clt_ctx_P smar_clt_ctx, sm_str_P mtrhs, ar_mtcnf_P ar_mtcnf)
{
	int err;
	uint ui;
	char *s;
#define AR_MT_RHS "mtrhs"

	SM_IS_SMAR_CLT_CTX(smar_clt_ctx);
	SM_IS_BUF(mtrhs);
	SM_REQUIRE(ar_mtcnf != NULL);

	ar_mtcnf->ar_mt_cnf_smc = sm_conf_new(AR_MT_RHS);
	if (NULL == ar_mtcnf->ar_mt_cnf_smc) {
		err = errno;

		sm_log_write(smar_clt_ctx->smac_ar_ctx->smar_lctx,
			AR_LCAT_CONFIG, AR_LMOD_CONFIG,
			SM_LOG_ERR, 8,
			"sev=ERROR, func=smar_mt_parse, sm_conf_new=NULL, errno=%d\n",
			err);
		return sm_error_temp(SM_EM_AR, ENOMEM);
	}

	err = sm_conf_read_data(ar_mtcnf->ar_mt_cnf_smc,
				(char *)sm_str_getdata(mtrhs),
				sm_str_getlen(mtrhs), true);
	if (err != 0) {
		sm_prt_conferr(AR_MT_RHS, ar_mtcnf->ar_mt_cnf_smc, err, smioerr);
		sm_log_write(smar_clt_ctx->smac_ar_ctx->smar_lctx,
			AR_LCAT_CONFIG, AR_LMOD_CONFIG,
			SM_LOG_ERR, 8,
			"sev=ERROR, func=smar_mt_parse, sm_conf_read_data=%m\n",
			err);
		sm_conf_destroy(ar_mtcnf->ar_mt_cnf_smc);
		ar_mtcnf->ar_mt_cnf_smc = NULL;
		return err;
	}

	err = sm_conf_scan(ar_mtcnf->ar_mt_cnf_smc, ar_mtrhs_defs, 0, ar_mtcnf);
	if (err != 0) {
		sm_log_write(smar_clt_ctx->smac_ar_ctx->smar_lctx,
			AR_LCAT_CONFIG, AR_LMOD_CONFIG,
			SM_LOG_ERR, 8,
			"sev=ERROR, func=smar_mt_parse, sm_conf_scan=%m\n",
			err);
		goto error;
	}

	/*
	**  HACK: "parse" configuration data.
	**  This will go away as soon as the configuration
	**  definition evolves... (array of IP addresses)
	*/

	while ((ui = ar_mtcnf->ar_mt_cnf_n_ips) < SMAR_MT_IP_MAX
		&& (s = ar_mtcnf->ar_mt_cnf_ips[ui]) != NULL)
	{
		err = sm_inet_a2ip(s, NULL, &ar_mtcnf->ar_mt_cnf_ipp[ui].ipp_addr);
		if (sm_is_err(err)) {
			sm_log_write(smar_clt_ctx->smac_ar_ctx->smar_lctx,
				AR_LCAT_CONFIG, AR_LMOD_CONFIG,
				SM_LOG_ERR, 8,
				"sev=ERROR, func=smar_mt_parse, mt_ip=%s, status=not_an_IP_address\n", s);
			goto error;
		}
		++ar_mtcnf->ar_mt_cnf_n_ips;
	}

	if (ar_mtcnf->ar_mt_cnf_n_ips > 0 && ar_mtcnf->ar_mt_cnf_host != NULL) {
		sm_log_write(smar_clt_ctx->smac_ar_ctx->smar_lctx,
			AR_LCAT_CONFIG, AR_LMOD_CONFIG,
			SM_LOG_ERR, 8,
			"sev=ERROR, func=smar_mt_parse, status=IP_address_and_hostname_defined\n");
			err = sm_err_perm(EINVAL);
	}

	if (0 == ar_mtcnf->ar_mt_cnf_port) {
		if (SMAR_PROT_LMTP == ar_mtcnf->ar_mt_cnf_prot &&
		    (ar_mtcnf->ar_mt_cnf_n_ips > 0 ||
			 ar_mtcnf->ar_mt_cnf_host != NULL))
		{
			sm_log_write(smar_clt_ctx->smac_ar_ctx->smar_lctx,
				AR_LCAT_CONFIG, AR_LMOD_CONFIG,
				SM_LOG_ERR, 8,
				"sev=ERROR, func=smar_mt_parse, protocol=lmtp, status=no_port_defined\n");
			err = sm_err_perm(EINVAL);
		}
#if 0
		else
			ar_mtcnf->ar_mt_cnf_port = SMTPC_PORT;
#endif
	}

	/* Sort entries XXX */
	(void) smar_mtipp_sort(ar_mtcnf);

/*
	sm_conf_prt_conf(ar_mtrhs_defs, ar_mtcnf->ar_mt_cnf_smc, smioout);
*/

  error:
	return err;
}
